@@ -3,32 +3,35 @@
 
     Import this module qualified to avoid clashing with the Prelude:
 
->>> import qualified Control.Foldl as L
+>>> import qualified Control.Foldl as Foldl
 
     Use 'fold' to apply a 'Fold' to a list:
 
->>> L.fold L.sum [1..100]
+>>> Foldl.fold Foldl.sum [1..100]
 5050
 
     'Fold's are 'Applicative's, so you can combine them using 'Applicative'
     combinators:
 
 >>> import Control.Applicative
->>> let average = (/) <$> L.sum <*> L.genericLength
+>>> let average = (/) <$> Foldl.sum <*> Foldl.genericLength
 
-    Taking the sum, the sum of squares, ..., upto the sum of x^5
+    … or you can use @do@ notation if you enable the @ApplicativeDo@ language
+    extension:
 
->>> import Data.Traversable
->>> let powerSums = sequenceA [L.premap (^n) L.sum | n <- [1..5]]
->>> L.fold powerSums [1..10]
-[55,385,3025,25333,220825]
+>>> :set -XApplicativeDo
+>>> let average = do total <- Foldl.sum; count <- Foldl.genericLength; return (total / count)
+
+    … or you can use the fact that the `Fold` type implements `Num` to do this:
+
+>>> let average = Foldl.sum / Foldl.genericLength
 
     These combined folds will still traverse the list only once, streaming
     efficiently over the list in constant space without space leaks:
 
->>> L.fold average [1..10000000]
+>>> Foldl.fold average [1..10000000]
 5000000.5
->>> L.fold ((,) <$> L.minimum <*> L.maximum) [1..10000000]
+>>> Foldl.fold ((,) <$> Foldl.minimum <*> Foldl.maximum) [1..10000000]
 (Just 1,Just 10000000)
 
     You might want to try enabling the @-flate-dmd-anal@ flag when compiling
@@ -38,7 +41,9 @@
 {-# LANGUAGE BangPatterns              #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE FlexibleContexts          #-}
+{-# LANGUAGE MultiParamTypeClasses     #-}
 {-# LANGUAGE RankNTypes                #-}
+{-# LANGUAGE ScopedTypeVariables       #-}
 {-# LANGUAGE Trustworthy               #-}
 
 module Control.Foldl (
@@ -99,7 +104,9 @@ module Control.Foldl (
     , set
     , hashSet
     , map
+    , foldByKeyMap
     , hashMap
+    , foldByKeyHashMap
     , vector
     , vectorM
 
@@ -151,19 +158,21 @@ import Control.Comonad
 import Data.Foldable (Foldable)
 import Data.Functor.Identity (Identity, runIdentity)
 import Data.Functor.Contravariant (Contravariant(..))
+import Data.HashMap.Strict (HashMap)
 import Data.Map.Strict (Map, alter)
 import Data.Maybe (fromMaybe)
 import Data.Monoid hiding ((<>))
-import Data.Semigroup (Semigroup(..))
 import Data.Semigroupoid (Semigroupoid)
+import Data.Functor.Extend (Extend(..))
 import Data.Profunctor
+import Data.Profunctor.Sieve
 import Data.Sequence ((|>))
 import Data.Vector.Generic (Vector, Mutable)
 import Data.Vector.Generic.Mutable (MVector)
 import Data.Hashable (Hashable)
 import Data.Traversable
 import Numeric.Natural (Natural)
-import System.Random.MWC (GenIO, createSystemRandom, uniformR)
+import System.Random (StdGen, newStdGen, uniformR)
 import Prelude hiding
     ( head
     , last
@@ -193,14 +202,13 @@ import qualified Data.Map.Strict             as Map
 import qualified Data.HashMap.Strict         as HashMap
 import qualified Data.HashSet                as HashSet
 import qualified Data.Vector.Generic         as V
+import qualified Control.Foldl.Util.Vector   as V
 import qualified Data.Vector.Generic.Mutable as M
-import qualified VectorBuilder.Builder
-import qualified VectorBuilder.Vector
 import qualified Data.Semigroupoid
 
 {- $setup
 
->>> import qualified Control.Foldl as L
+>>> import qualified Control.Foldl as Foldl
 
 >>> _2 f (x, y) = fmap (\i -> (x, i)) (f y)
 
@@ -236,8 +244,19 @@ instance Profunctor Fold where
     rmap = fmap
 
 instance Choice Fold where
-  right' (Fold step begin done) = Fold (liftA2 step) (Right begin) (fmap done)
-  {-# INLINE right' #-}
+    right' (Fold step begin done) = Fold (liftA2 step) (Right begin) (fmap done)
+    {-# INLINE right' #-}
+
+instance Cosieve Fold [] where
+    cosieve = fold
+    {-# INLINE cosieve #-}
+
+instance Costrong Fold where
+    unfirst p = fmap f list
+      where
+        f as = b
+          where (b, d) = fold p [ (a, d) | a <- as ]
+    {-# INLINE unfirst #-}
 
 instance Comonad (Fold a) where
     extract (Fold _ begin done) = done begin
@@ -257,6 +276,10 @@ instance Applicative (Fold a) where
         in  Fold step begin done
     {-# INLINE (<*>) #-}
 
+instance Extend (Fold a) where
+    duplicated = duplicate
+    {-# INLINE duplicated #-}
+
 instance Semigroup b => Semigroup (Fold a b) where
     (<>) = liftA2 (<>)
     {-# INLINE (<>) #-}
@@ -393,6 +416,10 @@ instance Applicative m => Applicative (FoldM m a) where
         in  FoldM step begin done
     {-# INLINE (<*>) #-}
 
+instance Monad m => Extend (FoldM m a) where
+    duplicated = duplicateM
+    {-# INLINE duplicated #-}
+
 instance Functor m => Profunctor (FoldM m) where
     rmap = fmap
     lmap f (FoldM step begin done) = FoldM step' begin done
@@ -517,7 +544,7 @@ foldM (FoldM step begin done) as0 = do
 
 {-| Convert a strict left 'Fold' into a scan
 
-    >>> L.scan L.length [1..5]
+    >>> Foldl.scan Foldl.length [1..5]
     [0,1,2,3,4,5]
 -}
 scan :: Fold a b -> [a] -> [b]
@@ -531,7 +558,7 @@ scan (Fold step begin done) as = foldr cons nil as begin
 
     \"Prescan\" means that the last element of the scan is not included
 
-    >>> L.prescan L.length [1..5]
+    >>> Foldl.prescan Foldl.length [1..5]
     [0,1,2,3,4]
 -}
 prescan :: Traversable t => Fold a b -> t a -> t b
@@ -548,7 +575,7 @@ prescan (Fold step begin done) as = bs
 
     \"Postscan\" means that the first element of the scan is not included
 
-    >>> L.postscan L.length [1..5]
+    >>> Foldl.postscan Foldl.length [1..5]
     [1,2,3,4,5]
 -}
 postscan :: Traversable t => Fold a b -> t a -> t b
@@ -795,14 +822,14 @@ random :: FoldM IO a (Maybe a)
 random = FoldM step begin done
   where
     begin = do
-        g <- createSystemRandom
+        g <- newStdGen
         return $! Pair3 g Nothing' (1 :: Int)
 
     step (Pair3 g Nothing'  _) a = return $! Pair3 g (Just' a) 2
     step (Pair3 g (Just' a) m) b = do
-        n <- uniformR (1, m) g
+        let (n, g') = uniformR (1, m) g
         let c = if n == 1 then b else a
-        return $! Pair3 g (Just' c) (m + 1)
+        return $! Pair3 g' (Just' c) (m + 1)
 
     done (Pair3 _ ma _) = return (lazy ma)
 {-# INLINABLE random #-}
@@ -813,7 +840,7 @@ data RandomNState v a = RandomNState
     { _size      ::                !VectorState
     , _reservoir ::                !(Mutable v RealWorld a)
     , _position  :: {-# UNPACK #-} !Int
-    , _gen       :: {-# UNPACK #-} !GenIO
+    , _gen       :: {-# UNPACK #-} !StdGen
     }
 
 -- | Pick several random elements, using reservoir sampling
@@ -829,15 +856,15 @@ randomN n = FoldM step begin done
         let s  = if n <= m' then Complete else Incomplete m'
         return $! RandomNState s mv (i + 1) g
     step (RandomNState  Complete      mv i g) a = do
-        r <- uniformR (0, i - 1) g
+        let (r, g') = uniformR (0, i - 1) g
         if r < n
             then M.unsafeWrite mv r a
             else return ()
-        return (RandomNState Complete mv (i + 1) g)
+        return (RandomNState Complete mv (i + 1) g')
 
     begin = do
         mv  <- M.new n
-        gen <- createSystemRandom
+        gen <- newStdGen
         let s = if n <= 0 then Complete else Incomplete 0
         return (RandomNState s mv 1 gen)
 
@@ -940,6 +967,28 @@ map = Fold step begin done
     done = id
 {-# INLINABLE map #-}
 
+
+{- | Given a 'Fold', produces a 'Map' which applies that fold to each @a@ separated by key @k@.
+
+>>> fold (foldByKeyMap Control.Foldl.sum) [("a",1), ("b",2), ("b",20), ("a",10)]
+fromList [("a",11),("b",22)]
+-}
+foldByKeyMap :: forall k a b. Ord k => Fold a b -> Fold (k, a) (Map k b)
+foldByKeyMap f = case f of
+  Fold (step0 :: x -> a -> x) (ini0 :: x) (end0 :: x -> b) ->
+    let
+      step :: Map k x -> (k,a) -> Map k x
+      step mp (k,a) = Map.alter addToMap k mp where
+        addToMap Nothing         = Just $ step0 ini0 a
+        addToMap (Just existing) = Just $ step0 existing a
+
+      ini :: Map k x
+      ini = Map.empty
+
+      end :: Map k x -> Map k b
+      end = fmap end0
+    in Fold step ini end where
+
 {-|
 Fold pairs into a hash-map.
 -}
@@ -951,15 +1000,30 @@ hashMap = Fold step begin done
     done = id
 {-# INLINABLE hashMap #-}
 
+{- | Given a 'Fold', produces a 'HashMap' which applies that fold to each @a@ separated by key @k@.
+
+>>> List.sort (HashMap.toList (fold (foldByKeyHashMap Control.Foldl.sum) [("a",1), ("b",2), ("b",20), ("a",10)]))
+[("a",11),("b",22)]
+-}
+foldByKeyHashMap :: forall k a b. (Hashable k, Eq k) => Fold a b -> Fold (k, a) (HashMap k b)
+foldByKeyHashMap f = case f of
+  Fold (step0 :: x -> a -> x) (ini0 :: x) (end0 :: x -> b) ->
+    let
+      step :: HashMap k x -> (k,a) -> HashMap k x
+      step mp (k,a) = HashMap.alter addToHashMap k mp where
+        addToHashMap Nothing         = Just $ step0 ini0 a
+        addToHashMap (Just existing) = Just $ step0 existing a
+
+      ini :: HashMap k x
+      ini = HashMap.empty
+
+      end :: HashMap k x -> HashMap k b
+      end = fmap end0
+    in Fold step ini end where
+
 -- | Fold all values into a vector
 vector :: Vector v a => Fold a (v a)
-vector = Fold step begin done
-  where
-    begin = VectorBuilder.Builder.empty
-
-    step x a = x <> VectorBuilder.Builder.singleton a
-
-    done = VectorBuilder.Vector.build
+vector = V.fromReverseListN <$> length <*> revList
 {-# INLINABLE vector #-}
 
 maxChunkSize :: Int
@@ -1126,10 +1190,10 @@ _Fold1 step = Fold step_ Nothing' lazy
 
 > fold (premap f folder) list = fold folder (List.map f list)
 
->>> fold (premap Sum L.mconcat) [1..10]
+>>> fold (premap Sum Foldl.mconcat) [1..10]
 Sum {getSum = 55}
 
->>> fold L.mconcat (List.map Sum [1..10])
+>>> fold Foldl.mconcat (List.map Sum [1..10])
 Sum {getSum = 55}
 
 > premap id = id
@@ -1185,8 +1249,6 @@ prefilter f (Fold step begin done) = Fold step' begin done
 
 {-| @(prefilterM f folder)@ returns a new 'FoldM' where the folder's input is used
   only when the input satisfies a monadic predicate f.
-
-> foldM (prefilterM p folder) list = foldM folder (filter p list)
 -}
 prefilterM :: (Monad m) => (a -> m Bool) -> FoldM m a r -> FoldM m a r
 prefilterM f (FoldM step begin done) = FoldM step' begin done
@@ -1219,10 +1281,10 @@ otherwise behaves the same as the original fold.
 
 > fold (drop n folder) list = fold folder (Data.List.genericDrop n list)
 
->>> L.fold (L.drop 3 L.sum) [10, 20, 30, 1, 2, 3]
+>>> Foldl.fold (Foldl.drop 3 Foldl.sum) [10, 20, 30, 1, 2, 3]
 6
 
->>> L.fold (L.drop 10 L.sum) [10, 20, 30, 1, 2, 3]
+>>> Foldl.fold (Foldl.drop 10 Foldl.sum) [10, 20, 30, 1, 2, 3]
 0
 -}
 
@@ -1240,10 +1302,10 @@ otherwise behaves the same as the original fold.
 
 > foldM (dropM n folder) list = foldM folder (Data.List.genericDrop n list)
 
->>> L.foldM (L.dropM 3 (L.generalize L.sum)) [10, 20, 30, 1, 2, 3]
+>>> Foldl.foldM (Foldl.dropM 3 (Foldl.generalize Foldl.sum)) [10, 20, 30, 1, 2, 3]
 6
 
->>> L.foldM (L.dropM 10 (L.generalize L.sum)) [10, 20, 30, 1, 2, 3]
+>>> Foldl.foldM (Foldl.dropM 10 (Foldl.generalize Foldl.sum)) [10, 20, 30, 1, 2, 3]
 0
 -}
 
@@ -1280,7 +1342,7 @@ type Handler a b =
 >>> fold (handles (filtered even) sum) [1..10]
 30
 
->>> fold (handles _2 L.mconcat) [(1,"Hello "),(2,"World"),(3,"!")]
+>>> fold (handles _2 Foldl.mconcat) [(1,"Hello "),(2,"World"),(3,"!")]
 "Hello World!"
 
 > handles id = id
@@ -1299,17 +1361,17 @@ handles k (Fold step begin done) = Fold step' begin done
 
 {- | @(foldOver f folder xs)@ folds all values from a Lens, Traversal, Prism or Fold with the given folder
 
->>> foldOver (_Just . both) L.sum (Just (2, 3))
+>>> foldOver (_Just . both) Foldl.sum (Just (2, 3))
 5
 
->>> foldOver (_Just . both) L.sum Nothing
+>>> foldOver (_Just . both) Foldl.sum Nothing
 0
 
-> L.foldOver f folder xs == L.fold folder (xs^..f)
+> Foldl.foldOver f folder xs == Foldl.fold folder (xs^..f)
 
-> L.foldOver (folded.f) folder == L.fold (handles f folder)
+> Foldl.foldOver (folded.f) folder == Foldl.fold (handles f folder)
 
-> L.foldOver folded == L.fold
+> Foldl.foldOver folded == Foldl.fold
 
 -}
 foldOver :: Handler s a -> Fold a b -> s -> b
@@ -1368,9 +1430,9 @@ handlesM k (FoldM step begin done) = FoldM step' begin done
 
 {- | @(foldOverM f folder xs)@ folds all values from a Lens, Traversal, Prism or Fold monadically with the given folder
 
-> L.foldOverM (folded.f) folder == L.foldM (handlesM f folder)
+> Foldl.foldOverM (folded.f) folder == Foldl.foldM (handlesM f folder)
 
-> L.foldOverM folded == L.foldM
+> Foldl.foldOverM folded == Foldl.foldM
 
 -}
 foldOverM :: Monad m => HandlerM m s a -> FoldM m a b -> s -> m b
@@ -1395,7 +1457,7 @@ folded k ts = contramap (\_ -> ()) (F.traverse_ k ts)
 >>> fold (handles (filtered even) sum) [1..10]
 30
 
->>> foldM (handlesM (filtered even) (L.mapM_ print)) [1..10]
+>>> foldM (handlesM (filtered even) (Foldl.mapM_ print)) [1..10]
 2
 4
 6
